@@ -16,6 +16,7 @@
 #include "common/code_environment.h"
 #include "common/debug.h"
 #include "common/signal.h"
+#include "common/io_priority.h"
 
 #include <dirent.h>
 #include <errno.h>
@@ -29,7 +30,10 @@
 
 
 Thread::Thread()
-  : thread_id(0)
+  : thread_id(0),
+    pid(0),
+    ioprio_class(-1),
+    ioprio_priority(-1)
 {
 }
 
@@ -38,40 +42,22 @@ Thread::~Thread()
 }
 
 void *Thread::_entry_func(void *arg) {
-  void *r = ((Thread*)arg)->entry();
+  void *r = ((Thread*)arg)->entry_wrapper();
   return r;
 }
 
-/**
- * Return the number of threads in this process.
- * So a single-threaded program would have one thread; a program with a main
- * thread and one child thread would have two threads, etc.
- */
-int Thread::get_num_threads(void)
+void *Thread::entry_wrapper()
 {
-  std::ostringstream oss;
-  oss << "/proc/" << getpid() << "/task";
-  DIR *dir = opendir(oss.str().c_str());
-  if (!dir) {
-    int err = errno;
-    return -err;
+  int p = ceph_gettid(); // may return -ENOSYS on other platforms
+  if (p > 0)
+    pid = p;
+  if (ioprio_class >= 0 &&
+      ioprio_priority >= 0) {
+    ceph_ioprio_set(IOPRIO_WHO_PROCESS,
+		    pid,
+		    IOPRIO_PRIO_VALUE(ioprio_class, ioprio_priority));
   }
-  int num_entries = 0;
-  while (true) {
-    struct dirent *e = readdir(dir);
-    if (!e)
-      break;
-    if ((strcmp(e->d_name, ".") == 0) ||
-        (strcmp(e->d_name, "..") == 0))
-      continue;
-    num_entries++;
-  }
-  ::closedir(dir);
-  if (num_entries == 0) {
-    // Shouldn't happen.
-    return -EINVAL;
-  }
-  return num_entries;
+  return entry();
 }
 
 const pthread_t &Thread::get_thread_id()
@@ -103,6 +89,8 @@ int Thread::try_create(size_t stacksize)
   stacksize &= CEPH_PAGE_MASK;  // must be multiple of page
   if (stacksize) {
     thread_attr = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
+    if (!thread_attr)
+      return -ENOMEM;
     pthread_attr_init(thread_attr);
     pthread_attr_setstacksize(thread_attr, stacksize);
   }
@@ -158,3 +146,15 @@ int Thread::detach()
 {
   return pthread_detach(thread_id);
 }
+
+int Thread::set_ioprio(int cls, int prio)
+{
+  // fixme, maybe: this can race with create()
+  ioprio_class = cls;
+  ioprio_priority = prio;
+  if (pid && cls >= 0 && prio >= 0)
+    return ceph_ioprio_set(IOPRIO_WHO_PROCESS,
+			   pid,
+			   IOPRIO_PRIO_VALUE(cls, prio));
+  return 0;
+}
