@@ -103,6 +103,10 @@
 #define MSG_OSD_BACKFILL_RESERVE 99
 #define MSG_OSD_RECOVERY_RESERVE 150
 
+#define MSG_OSD_PG_PUSH        105
+#define MSG_OSD_PG_PULL        106
+#define MSG_OSD_PG_PUSH_REPLY  107
+
 // *** MDS ***
 
 #define MSG_MDS_BEACON             100  // to monitor
@@ -125,6 +129,8 @@
 #define MSG_MDS_DENTRYLINK         0x20c
 #define MSG_MDS_FINDINO            0x20d
 #define MSG_MDS_FINDINOREPLY       0x20e
+#define MSG_MDS_OPENINO            0x20f
+#define MSG_MDS_OPENINOREPLY       0x210
 
 #define MSG_MDS_LOCK               0x300
 #define MSG_MDS_INODEFILECAPS      0x301
@@ -160,19 +166,24 @@
 
 class Messenger;
 
-struct Connection : public RefCountedObject {
+struct Connection : private RefCountedObject {
   Mutex lock;
   Messenger *msgr;
   RefCountedObject *priv;
   int peer_type;
   entity_addr_t peer_addr;
+  utime_t last_keepalive_ack;
+private:
   uint64_t features;
+public:
   RefCountedObject *pipe;
   bool failed;              /// true if we are a lossy connection that has failed.
 
   int rx_buffers_version;
   map<tid_t,pair<bufferlist,int> > rx_buffers;
 
+  friend class boost::intrusive_ptr<Connection>;
+
 public:
   Connection(Messenger *m)
     : lock("Connection::lock"),
@@ -183,6 +194,9 @@ public:
       pipe(NULL),
       failed(false),
       rx_buffers_version(0) {
+    // we are managed exlusively by ConnectionRef; make it so you can
+    //   ConnectionRef foo = new Connection;
+    nref.set(0);
   }
   ~Connection() {
     //generic_dout(0) << "~Connection " << this << dendl;
@@ -194,10 +208,6 @@ public:
       pipe->put();
   }
 
-  Connection *get() {
-    return static_cast<Connection *>(RefCountedObject::get());
-  }
-
   void set_priv(RefCountedObject *o) {
     Mutex::Locker l(lock);
     if (priv)
@@ -229,13 +239,15 @@ public:
     }
     return !failed;
   }
-  void clear_pipe(RefCountedObject *old_p) {
+  bool clear_pipe(RefCountedObject *old_p) {
     if (old_p == pipe) {
       Mutex::Locker l(lock);
       pipe->put();
       pipe = NULL;
       failed = true;
+      return true;
     }
+    return false;
   }
   void reset_pipe(RefCountedObject *p) {
     Mutex::Locker l(lock);
@@ -277,6 +289,10 @@ public:
     Mutex::Locker l(lock);
     rx_buffers.erase(tid);
   }
+
+  utime_t get_last_keepalive_ack() const {
+    return last_keepalive_ack;
+  }
 };
 typedef boost::intrusive_ptr<Connection> ConnectionRef;
 
@@ -303,7 +319,7 @@ protected:
   /* time at which message was fully read */
   utime_t recv_complete_stamp;
 
-  Connection *connection;
+  ConnectionRef connection;
 
   // release our size in bytes back to this throttler when our payload
   // is adjusted or when we are destroyed.
@@ -351,18 +367,14 @@ public:
 protected:
   virtual ~Message() { 
     assert(nref.read() == 0);
-    if (connection)
-      connection->put();
     if (byte_throttler)
       byte_throttler->put(payload.length() + middle.length() + data.length());
     if (msg_throttler)
       msg_throttler->put();
   }
 public:
-  Connection *get_connection() { return connection; }
-  void set_connection(Connection *c) {
-    if (connection)
-      connection->put();
+  const ConnectionRef& get_connection() { return connection; }
+  void set_connection(const ConnectionRef& c) {
     connection = c;
   }
   void set_byte_throttler(Throttle *t) { byte_throttler = t; }
