@@ -3,12 +3,9 @@
 /*
  * Ceph - scalable distributed file system
  *
- * Copyright (C) 2004-2006 Sage Weil <sage@newdream.net>
+ * Copyright (C) 2004-2012 Sage Weil <sage@newdream.net> and others
  *
- * This is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License version 2, as published by the Free Software
- * Foundation.  See file COPYING.
+ * LGPL2.  See file COPYING.
  *
  */
 
@@ -248,6 +245,8 @@ static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag,
 {
   std::vector<string> names;
   int r = rbd.list(io_ctx, names);
+  if (r == -ENOENT)
+    r = 0;
   if (r < 0)
     return r;
 
@@ -506,6 +505,10 @@ static int do_show_info(const char *imgname, librbd::Image& image,
       return r;
   }
 
+  char prefix[RBD_MAX_BLOCK_NAME_SIZE + 1];
+  strncpy(prefix, info.block_name_prefix, RBD_MAX_BLOCK_NAME_SIZE);
+  prefix[RBD_MAX_BLOCK_NAME_SIZE] = '\0';
+
   if (f) {
     f->open_object_section("image");
     f->dump_string("name", imgname);
@@ -513,7 +516,7 @@ static int do_show_info(const char *imgname, librbd::Image& image,
     f->dump_unsigned("objects", info.num_objs);
     f->dump_int("order", info.order);
     f->dump_unsigned("object_size", info.obj_size);
-    f->dump_string("block_name_prefix", info.block_name_prefix);
+    f->dump_string("block_name_prefix", prefix);
     f->dump_int("format", (old_format ? 1 : 2));
   } else {
     cout << "rbd image '" << imgname << "':\n"
@@ -523,7 +526,7 @@ static int do_show_info(const char *imgname, librbd::Image& image,
 	 << "\torder " << info.order
 	 << " (" << prettybyte_t(info.obj_size) << " objects)"
 	 << std::endl
-	 << "\tblock_name_prefix: " << info.block_name_prefix
+	 << "\tblock_name_prefix: " << prefix
 	 << std::endl
 	 << "\tformat: " << (old_format ? "1" : "2")
 	 << std::endl;
@@ -868,6 +871,11 @@ void rbd_bencher_completion(void *vc, void *pc)
   librbd::RBD::AioCompletion *c = (librbd::RBD::AioCompletion *)vc;
   rbd_bencher *b = static_cast<rbd_bencher *>(pc);
   //cout << "complete " << c << std::endl;
+  int ret = c->get_return_value();
+  if (ret != 0) {
+    cout << "write error: " << cpp_strerror(ret) << std::endl;
+    assert(0 == ret);
+  }
   b->lock.Lock();
   b->in_flight--;
   b->cond.Signal();
@@ -2553,15 +2561,8 @@ if (!set_conf_param(v, p1, p2, p3)) { \
   case OPT_LIST:
     r = do_list(rbd, io_ctx, lflag, formatter.get());
     if (r < 0) {
-      switch (r) {
-      case -ENOENT:
-        cerr << "rbd: pool " << poolname << " doesn't contain rbd images"
-	     << std::endl;
-        break;
-      default:
-        cerr << "rbd: list: " << cpp_strerror(-r) << std::endl;
-      }
-      return EXIT_FAILURE;
+      cerr << "rbd: list: " << cpp_strerror(-r) << std::endl;
+      return -r;
     }
     break;
 
